@@ -8,10 +8,29 @@ import aiohttp
 from loguru import logger
 from websockets.asyncio.client import connect, ClientConnection
 from websockets.exceptions import ConnectionClosed
+
 from ._cfg import *
 
 
 class GameClient:
+    """
+    A game client for connecting to and interacting with Goodgame Empire game server via WebSocket.
+    
+    This client handles connection management, authentication, message sending/receiving,
+    and provides various utility methods for game interactions.
+    
+    Attributes:
+        url (str): The WebSocket server URL to connect to
+        server_header (str): Server identifier header for messages
+        username (str): Username for authentication
+        password (str): Password for authentication
+        ws (Optional[ClientConnection]): The WebSocket connection object
+        connected (asyncio.Event): Event indicating connection status
+        user_agent (str): Random user agent string from default list
+    
+    Types:
+        HandlerType: Callable handler type for async/sync message handlers
+    """
     
     HandlerType = Callable[[Any], Union[Any, Awaitable[Any]]]
     
@@ -22,6 +41,15 @@ class GameClient:
         username: str,
         password: str
     ) -> None:
+        """
+        Initialize the game client.
+        
+        Args:
+            url: The WebSocket server URL
+            server_header: Server identifier for message routing
+            username: Game account username
+            password: Game account password
+        """
         
         self.url = url
         self.server_header = server_header
@@ -36,27 +64,43 @@ class GameClient:
         self._msg_queue: asyncio.Queue[Dict[str, Any]] = asyncio.Queue()
         self._pending_futures: Dict[str, List[asyncio.Future]] = {}
         self.user_agent = random.choice(DEFAULT_UA_LIST)
-        
+   
     async def connect(self) -> None:
         """
         Main client loop to manage connection and reconnection.
+        
+        Continuously attempts to connect to the server and automatically reconnects
+        on connection loss unless explicitly stopped. Handles various connection
+        exceptions and implements exponential backoff for reconnections.
         """
         while not self._stop_event.is_set():
             try:
                 await self._run_connection_session()
-            except ConnectionClosed:
-                logger.warning("Connection closed, attempting to reconnect in 5 seconds...")
+            except (ConnectionClosed, asyncio.CancelledError):
+                logger.warning("Connection closed gracefully.")
                 self.connected.clear()
+                if self._stop_event.is_set():
+                    break
+                logger.info("Attempting to reconnect in 5 seconds...")
                 await asyncio.sleep(5)
             except Exception as e:
-                logger.error(f"Error during connection session: {e}, attempting to reconnect in 5 seconds...")
+                logger.error(f"Error during connection session: {e}")
                 self.connected.clear()
+                if self._stop_event.is_set():
+                    break
+                logger.info("Attempting to reconnect in 5 seconds...")
                 await asyncio.sleep(5)
         logger.info("Client shutdown complete.")
 
     async def _run_connection_session(self) -> None:
         """
         Handles a single connection session.
+        
+        Establishes WebSocket connection, starts background tasks (listener, 
+        keep-alive, nch), initializes the connection, and manages task lifecycle.
+        
+        Raises:
+            Exception: Re-raises exceptions from failed background tasks
         """
         async with connect(
             self.url,
@@ -83,18 +127,42 @@ class GameClient:
                 self._tasks,
                 return_when=asyncio.FIRST_EXCEPTION
             )
-            for t in pending: t.cancel()
+            
+            for t in pending:
+                t.cancel()
+            
+            await asyncio.gather(*pending, return_exceptions=True)
+            
             for t in done:
-                if t.exception(): raise t.exception()
-                
+                exc = t.exception()
+                if exc and not isinstance(exc, asyncio.CancelledError):
+                    raise exc
+   
     async def _init(self) -> bool:
+        """
+        Initialize socket connection and perform login.
+        
+        Returns:
+            bool: True if initialization and login were successful
+        """
         await self._init_socket()
         return await self.login(self.username, self.password)            
-                
+ 
     async def run_jobs(self) -> None:
+        """
+        Placeholder for running client jobs/tasks.
+        
+        This method can be overridden by subclasses to implement specific
+        game logic or periodic tasks.
+        """
         pass
 
     async def disconnect(self) -> None:
+        """
+        Disconnect from the server and clean up resources.
+        
+        Closes the WebSocket connection and updates connection status.
+        """
         self.connected.clear()
         if self.ws:
             try:
@@ -103,37 +171,103 @@ class GameClient:
                 logger.error(f"Error closing connection: {e}")
 
         logger.info("Disconnected!.")
-        
+ 
     async def shutdown(self) -> None:
+        """
+        Gracefully shutdown the client.
+        
+        Stops all background tasks, disconnects from server, and cancels
+        all pending operations.
+        """
         self._stop_event.set()
         await self.disconnect()
         
-        for task in self._tasks:
+        current_task = asyncio.current_task()
+        tasks_to_cancel = [t for t in self._tasks if t is not current_task]
+
+        for task in tasks_to_cancel:
             task.cancel()
-        await asyncio.gather(*self._tasks, return_exceptions=True)
-        
+            
+        await asyncio.gather(*tasks_to_cancel, return_exceptions=True)
+
     async def send(self, message: str) -> None:
+        """
+        Send a raw message through the WebSocket connection.
+        
+        Args:
+            message: The message string to send
+            
+        Raises:
+            RuntimeError: If client is not connected
+        """
         if not self.ws: raise RuntimeError("GGClient not connected!")
         await self.ws.send(message)
-            
+ 
     async def send_message(self, parts: List[str]) -> None:
+        """
+        Send a formatted message with percent-separated parts.
+        
+        Args:
+            parts: List of message components to join with '%' separators
+        """
         msg = "%".join(["", *parts, ""])
         await self.send(msg)
-        
+ 
     async def send_raw_message(self, command: str, data: List[Any]) -> None:
+        """
+        Send a raw command message with JSON-serialized data.
+        
+        Args:
+            command: The command identifier
+            data: List of data elements to send (dicts/lists will be JSON serialized)
+        """
         json_parts = [json.dumps(item) if isinstance(item, (dict, list)) else item for item in data]
         await self.send_message(["xt", self.server_header, command, "1", *json_parts])
 
     async def send_json_message(self, command: str, data: Dict[str, Any]) -> None:
+        """
+        Send a JSON-formatted command message.
+        
+        Args:
+            command: The command identifier
+            data: Dictionary data to send as JSON
+        """
         await self.send_message(["xt", self.server_header, command, "1", json.dumps(data)])
 
     async def send_xml_message(self, t: str, action: str, r: str, data: str) -> None:
-        await self.send(f"<msg t='{t}'><body action='{action}' r='{r}'>{data}</body></msg>")
+        """
+        Send an XML-formatted message.
         
+        Args:
+            t: Message type attribute
+            action: Action attribute
+            r: R attribute (typically room ID)
+            data: XML content as string
+        """
+        await self.send(f"<msg t='{t}'><body action='{action}' r='{r}'>{data}</body></msg>")
+ 
     async def receive(self) -> Dict[str, Any]:
+        """
+        Receive the next message from the message queue.
+        
+        Returns:
+            Dict[str, Any]: Parsed message dictionary
+        """
         return await self._msg_queue.get()
-    
+
     def _parse_message(self, message: str) -> Dict[str, Any]:
+        """
+        Parse incoming message into structured format.
+        
+        Handles both XML and JSON message formats. XML messages are parsed
+        using regex, while JSON messages are split and parsed accordingly.
+        
+        Args:
+            message: Raw message string from server
+            
+        Returns:
+            Dict[str, Any]: Parsed message with type and payload
+        """
         if message.startswith("<"):
             m = re.search(r"<msg t='(.*?)'><body action='(.*?)' r='(.*?)'>(.*?)</body></msg>", message)
             t_val, action, r_val, data = m.groups()
@@ -150,8 +284,16 @@ class GameClient:
         
         parsed_data = {"type": "json", "payload": {"command": cmd, "status": status, "data": data}}      
         return parsed_data
-    
+
     async def _listener(self) -> None:
+        """
+        Background task to listen for incoming messages.
+        
+        Continuously receives messages from WebSocket, parses them, and:
+        - Adds to message queue
+        - Resolves pending futures waiting for specific commands
+        - Calls registered handler methods (on_* pattern)
+        """
         try:
             async for raw in self.ws:
                 text = raw.decode('utf-8') if isinstance(raw, bytes) else raw
@@ -176,8 +318,23 @@ class GameClient:
                         handler(data)
         except ConnectionClosed:
             logger.warning("Connection closed, stopping listener...")
-            
+        except asyncio.CancelledError:
+            logger.warning("Listener task cancelled.")
+
     async def wait_for_response(self, command: str, timeout: float = 5.0) -> Any:
+        """
+        Wait for a specific command response with timeout.
+        
+        Args:
+            command: The command to wait for
+            timeout: Maximum time to wait in seconds
+            
+        Returns:
+            Any: Response data for the command
+            
+        Raises:
+            asyncio.TimeoutError: If timeout is reached without response
+        """
         deadline = asyncio.get_event_loop().time() + timeout
         buffer: List[Dict[str, Any]] = []
         try:
@@ -195,38 +352,94 @@ class GameClient:
         finally:
             for m in buffer:
                 await self._msg_queue.put(m)
-            
+
     async def send_rpc(self, command: str, data: Dict[str, Any], timeout: float = 5.0) -> Any:
+        """
+        Send RPC command and wait for response.
+        
+        Args:
+            command: RPC command name
+            data: RPC parameters as dictionary
+            timeout: Response timeout in seconds
+            
+        Returns:
+            Any: Response data from server
+        """
         await self.send_json_message(command, data)
         return await self.wait_for_response(command, timeout)
-    
+
     async def send_hrpc(self, command: str, data: Dict[str, Any], handler: HandlerType, timeout: float = 5.0) -> Any:
+        """
+        Send RPC command and process response with handler.
+        
+        Args:
+            command: RPC command name
+            data: RPC parameters as dictionary
+            handler: Callback function to process response
+            timeout: Response timeout in seconds
+        """
         await self.send_json_message(command, data)
         resp_data = await self.wait_for_response(command, timeout)
         to_handle = handler(resp_data)
         if inspect.isawaitable(to_handle):
             await to_handle
-            
+
     async def keep_alive(self, interval: int = 60) -> None:
-        await self.connected.wait()
-        while self.connected.is_set() and not self._stop_event.is_set():
-            await asyncio.sleep(interval)
-            await self.send_raw_message("pin", ["<RoundHouseKick>"])
-            
-    async def _nch(self, interval: int = 360) -> None:
-        await self.connected.wait()
-        while self.connected.is_set():
-            await asyncio.sleep(interval)
-            await self.send(f'%xt%{self.server_header}%nch%1%')
+        """
+        Send periodic keep-alive messages to maintain connection.
         
+        Args:
+            interval: Time between keep-alive messages in seconds
+        """
+        try:
+            await self.connected.wait()
+            while self.connected.is_set() and not self._stop_event.is_set():
+                await asyncio.sleep(interval)
+                await self.send_raw_message("pin", ["<RoundHouseKick>"])
+        except asyncio.CancelledError:
+            logger.warning("Keep-alive task cancelled.")
+
+    async def _nch(self, interval: int = 360) -> None:
+        """
+        Send periodic NCH (likely "noop" or heartbeat) messages.
+        
+        Args:
+            interval: Time between NCH messages in seconds
+        """
+        try:
+            await self.connected.wait()
+            while self.connected.is_set():
+                await asyncio.sleep(interval)
+                await self.send(f'%xt%{self.server_header}%nch%1%')
+        except asyncio.CancelledError:
+            logger.warning("NCH task cancelled.")
+
     async def _init_socket(self):
+        """
+        Initialize socket connection with handshake messages.
+        
+        Sends version check, login, auto-join, and round-trip messages
+        to establish the connection protocol.
+        """
         await self.send_xml_message("sys", "verChk", "0", "<ver v='166' />")
         await self.send_xml_message("sys", "login", "0", 
                                         f"<login z='{self.server_header}'><nick><![CDATA[]]></nick><pword><![CDATA[1123010%fr%0]]></pword></login>")
         await self.send_xml_message("sys", "autoJoin", "-1", "")
         await self.send_xml_message("sys", "roundTrip", "1", "")
-            
+
     async def fetch_game_db(self) -> dict:
+        """
+        Fetch the game database/items definition.
+        
+        Retrieves current game version and corresponding items database
+        from the game servers.
+        
+        Returns:
+            dict: Game items database as dictionary
+            
+        Raises:
+            aiohttp.ClientError: If HTTP request fails
+        """
         async with aiohttp.ClientSession() as session:
             async with session.get(GAME_VERSION_URL) as resp:
                 resp.raise_for_status()
@@ -239,15 +452,28 @@ class GameClient:
                 db_resp.raise_for_status()
                 data = await db_resp.json()
                 return data
-            
+
     async def login(
         self,
         username: str,
         password: str
         ) -> bool:
+        """
+        Authenticate with the game server.
         
+        Args:
+            username: Game account username
+            password: Game account password
+
+        Returns:
+            bool: True if login successful, False if failed or server cooldown
+            
+        Note:
+            Handles server cooldowns by automatically waiting and retrying
+        """
         if not self.connected.is_set():
             logger.error("Not connected yet!")
+            return False
             
         while True:
             try:
@@ -281,14 +507,14 @@ class GameClient:
                     logger.warning("Wrong username or password!")
                     return False
                 
-                if isinstance(response, dict):
+                if isinstance(response, dict) and "CD" in response:
                     cooldown_value = response["CD"]
                     logger.debug(f'Connection locked by the server! Reconnect in {cooldown_value} sec!')
                     await asyncio.sleep(cooldown_value)
                     
+                else:
+                    return True
+                 
             except Exception as e:
-                logger.error(e)
-                return False
-            
-            logger.error("Unexpected response in login status!")
-            return False
\ No newline at end of file
+                logger.error(f"Error during login: {e}")
+                return False
\ No newline at end of file
