@@ -1,3 +1,5 @@
+from .account.account import Account
+from .account.auth import Auth
 from .castle.castle import Castle
 from .castle.defense import Defense
 from .map.bookmarks import Bookmarks
@@ -11,6 +13,8 @@ from .misc.quests import Quests
 from .misc.tax import Tax
 from .misc.global_effects import GlobalEffects
 from .misc.tutorial import Tutorial
+from .misc.gifts import Gifts
+from .presets.presets import Presets
 from .army.lords import Lords
 from .building.buildings import Buildings
 from .building.repair import Repair
@@ -41,6 +45,7 @@ from .events.imperial_patronage import ImperialPatronage
  
 
 class GGXS(
+    Auth,
     Castle,
     Bookmarks,
     Map,
@@ -74,7 +79,10 @@ class GGXS(
     Quests,
     Tax,
     GlobalEffects,
-    Tutorial
+    Tutorial,
+    Gifts,
+    Account,
+    Presets
 ):
     
     
